@@ -46,6 +46,11 @@
 #include "slurm/slurm_errno.h"
 #include "src/common/slurm_xlator.h"
 
+#include "src/common/pack.h"
+#include "src/common/slurm_protocol_api.h"
+#include "src/common/xmalloc.h"
+#include "src/common/xstring.h"
+
 /*
  * These variables are required by the generic plugin interface.  If they
  * are not found in the plugin, the plugin loader will ignore it.
@@ -75,6 +80,7 @@ const char plugin_name[] = "Null authentication plugin";
 const char plugin_type[] = "auth/none";
 const uint32_t plugin_id = AUTH_PLUGIN_NONE;
 const uint32_t plugin_version = SLURM_VERSION_NUMBER;
+const bool hash_enable = false;
 
 /*
  * An opaque type representing authentication credentials.  This type can be
@@ -138,7 +144,8 @@ extern int fini(void)
  * Allocate and initializes a credential.  This function should return
  * NULL if it cannot allocate a credential.
  */
-auth_credential_t *auth_p_create(char *auth_info)
+auth_credential_t *auth_p_create(char *auth_info, uid_t r_uid,
+				 void *data, int dlen)
 {
 	auth_credential_t *cred = xmalloc(sizeof(*cred));
 
@@ -154,15 +161,13 @@ auth_credential_t *auth_p_create(char *auth_info)
  * Free a credential that was allocated with auth_p_create() or
  * auth_p_unpack().
  */
-int auth_p_destroy(auth_credential_t *cred)
+extern void auth_p_destroy(auth_credential_t *cred)
 {
-	if (!cred) {
-		slurm_seterrno(ESLURM_AUTH_MEMORY);
-		return SLURM_ERROR;
-	}
+	if (!cred)
+		return;
+
 	xfree(cred->hostname);
 	xfree(cred);
-	return SLURM_SUCCESS;
 }
 
 /*
@@ -175,32 +180,16 @@ int auth_p_verify(auth_credential_t *cred, char *auth_info)
 	return SLURM_SUCCESS;
 }
 
-/*
- * Obtain the Linux UID from the credential.  The accuracy of this data
- * is not assured until auth_p_verify() has been called for it.
- */
-uid_t auth_p_get_uid(auth_credential_t *cred)
+extern void auth_p_get_ids(auth_credential_t *cred, uid_t *uid, gid_t *gid)
 {
 	if (!cred) {
-		slurm_seterrno(ESLURM_AUTH_BADARG);
-		return SLURM_AUTH_NOBODY;
+		*uid = SLURM_AUTH_NOBODY;
+		*gid = SLURM_AUTH_NOBODY;
+		return;
 	}
 
-	return cred->uid;
-}
-
-/*
- * Obtain the Linux GID from the credential.
- * See auth_p_get_uid() above for details on correct behavior.
- */
-gid_t auth_p_get_gid(auth_credential_t *cred)
-{
-	if (!cred) {
-		slurm_seterrno(ESLURM_AUTH_BADARG);
-		return SLURM_AUTH_NOBODY;
-	}
-
-	return cred->gid;
+	*uid = cred->uid;
+	*gid = cred->gid;
 }
 
 /*
@@ -217,6 +206,29 @@ char *auth_p_get_host(auth_credential_t *cred)
 	return xstrdup(cred->hostname);
 }
 
+extern int auth_p_get_data(auth_credential_t *cred, char **data, uint32_t *len)
+{
+	if (!cred) {
+		slurm_seterrno(ESLURM_AUTH_BADARG);
+		return SLURM_ERROR;
+	}
+
+	*data = NULL;
+	*len = 0;
+
+	return SLURM_SUCCESS;
+}
+
+extern void *auth_p_get_identity(auth_credential_t *cred)
+{
+	if (!cred) {
+		slurm_seterrno(ESLURM_AUTH_BADARG);
+		return NULL;
+	}
+
+	return NULL;
+}
+
 /*
  * Marshall a credential for transmission over the network, according to
  * Slurm's marshalling protocol.
@@ -229,8 +241,8 @@ int auth_p_pack(auth_credential_t *cred, buf_t *buf, uint16_t protocol_version)
 	}
 
 	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		pack32((uint32_t) cred->uid, buf);
-		pack32((uint32_t) cred->gid, buf);
+		pack32(cred->uid, buf);
+		pack32(cred->gid, buf);
 		packstr(cred->hostname, buf);
 	} else {
 		error("%s: Unknown protocol version %d",
@@ -248,8 +260,6 @@ int auth_p_pack(auth_credential_t *cred, buf_t *buf, uint16_t protocol_version)
 auth_credential_t *auth_p_unpack(buf_t *buf, uint16_t protocol_version)
 {
 	auth_credential_t *cred = NULL;
-	uint32_t tmpint;
-	uint32_t uint32_tmp = 0;
 
 	if (!buf) {
 		slurm_seterrno(ESLURM_AUTH_BADARG);
@@ -260,20 +270,9 @@ auth_credential_t *auth_p_unpack(buf_t *buf, uint16_t protocol_version)
 	cred = xmalloc(sizeof(*cred));
 
 	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		/*
-		 * We do it the hard way because we don't know anything about
-		 * the size of uid_t or gid_t, only that they are integer
-		 * values.  We pack them as 32-bit integers, but we can't pass
-		 * addresses to them directly to unpack as 32-bit integers
-		 * because there will be bad clobbering if they really aren't.
-		 * This technique ensures a warning at compile time if the sizes
-		 * are incompatible.
-		 */
-		safe_unpack32(&tmpint, buf);
-		cred->uid = tmpint;
-		safe_unpack32(&tmpint, buf);
-		cred->gid = tmpint;
-		safe_unpackstr_xmalloc(&cred->hostname, &uint32_tmp, buf);
+		safe_unpack32(&cred->uid, buf);
+		safe_unpack32(&cred->gid, buf);
+		safe_unpackstr(&cred->hostname, buf);
 	} else {
 		error("%s: unknown protocol version %u",
 		      __func__, protocol_version);
