@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  parsers.c - Slurm data parsers
  *****************************************************************************
- *  Copyright (C) 2022 SchedMD LLC.
- *  Written by Nathan Rini <nate@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -196,6 +195,7 @@ static data_for_each_cmd_t _foreach_flag_parser(data_t *src, void *arg)
 	const parser_t *const parser = args->parser;
 	char *path = NULL;
 	bool matched_any = false;
+	data_for_each_cmd_t rc = DATA_FOR_EACH_CONT;
 
 	xassert(args->magic == MAGIC_FOREACH_LIST_FLAG);
 	xassert(args->args->magic == MAGIC_ARGS);
@@ -203,6 +203,13 @@ static data_for_each_cmd_t _foreach_flag_parser(data_t *src, void *arg)
 
 	path = _flag_parent_path(&path, args);
 
+	if (data_convert_type(src, DATA_TYPE_STRING) != DATA_TYPE_STRING) {
+		on_warn(PARSING, parser->type, args->args, path,
+			__func__, "Ignoring unexpected field of type %s",
+			data_get_type_string(src));
+		goto cleanup;
+	}
+
 	for (int8_t i = 0; (i < parser->flag_bit_array_count); i++) {
 		const flag_bit_t *bit = &parser->flag_bit_array[i];
 		bool matched = !xstrcasecmp(data_get_string(src), bit->name);
@@ -226,18 +233,15 @@ static data_for_each_cmd_t _foreach_flag_parser(data_t *src, void *arg)
 
 	}
 
-	args->index++;
-
-	if (!matched_any) {
+	if (!matched_any)
 		on_error(PARSING, parser->type, args->args,
 			 ESLURM_DATA_FLAGS_INVALID, path, __func__,
 			 "Unknown flag \"%s\"", data_get_string(src));
-		xfree(path);
-		return DATA_FOR_EACH_FAIL;
-	}
 
+cleanup:
 	xfree(path);
-	return DATA_FOR_EACH_CONT;
+	args->index++;
+	return rc;
 }
 
 static int _parse_flag(void *dst, const parser_t *const parser, data_t *src,
@@ -446,6 +450,8 @@ static data_for_each_cmd_t _foreach_array_entry(data_t *src, void *arg)
 	foreach_nt_array_t *args = arg;
 	void *obj = NULL;
 	data_t *ppath = NULL;
+	const parser_t *const parser = args->parser;
+	const parser_t *const array_parser = args->array_parser;
 
 	xassert(args->magic == MAGIC_FOREACH_NT_ARRAY);
 	xassert((args->index > 0) || (args->index == -1));
@@ -464,21 +470,22 @@ static data_for_each_cmd_t _foreach_array_entry(data_t *src, void *arg)
 				    data_get_string(ppath_last), args->index);
 	}
 
-	if (args->parser->model == PARSER_MODEL_NT_PTR_ARRAY)
-		obj = alloc_parser_obj(args->parser);
-	else if (args->parser->model == PARSER_MODEL_NT_ARRAY)
-		obj = args->sarray + (args->parser->size * args->index);
+	if (array_parser->model == PARSER_MODEL_NT_PTR_ARRAY)
+		obj = alloc_parser_obj(parser);
+	else if (array_parser->model == PARSER_MODEL_NT_ARRAY)
+		obj = args->sarray + (parser->size * args->index);
 
-	if ((rc = parse(obj, NO_VAL, args->parser, src, args->args, ppath))) {
+	if ((rc = parse(obj, NO_VAL, parser, src, args->args, ppath))) {
 		log_flag(DATA, "%s object at 0x%"PRIxPTR" freed due to parser error: %s",
-			 args->parser->obj_type_string, (uintptr_t) obj,
+			 parser->obj_type_string, (uintptr_t) obj,
 			 slurm_strerror(rc));
-		free_parser_obj(args->parser, obj);
+		if (array_parser->model == PARSER_MODEL_NT_PTR_ARRAY)
+			free_parser_obj(parser, obj);
 		FREE_NULL_DATA(ppath);
 		return DATA_FOR_EACH_FAIL;
 	}
 
-	if (args->parser->model == PARSER_MODEL_NT_PTR_ARRAY) {
+	if (array_parser->model == PARSER_MODEL_NT_PTR_ARRAY) {
 		xassert(!args->array[args->index]);
 		args->array[args->index] = obj;
 	}
@@ -496,7 +503,7 @@ static int _parse_nt_array(const parser_t *const parser, void *dst, data_t *src,
 	foreach_nt_array_t fargs = {
 		.magic = MAGIC_FOREACH_NT_ARRAY,
 		.array_parser = parser,
-		.parser = find_parser_by_type(parser->pointer_type),
+		.parser = find_parser_by_type(parser->array_type),
 		.args = args,
 		.parent_path = parent_path,
 		.index = -1,
@@ -521,7 +528,11 @@ static int _parse_nt_array(const parser_t *const parser, void *dst, data_t *src,
 				      sizeof(*fargs.array));
 	else if (parser->model == PARSER_MODEL_NT_ARRAY)
 		fargs.sarray = xcalloc(data_get_list_length(src) + 1,
-				       sizeof(fargs.parser->size));
+				       fargs.parser->size);
+
+	/* verify new array actually allocated */;
+	xassert((fargs.array && (xsize(fargs.array) > 0)) ^
+		(fargs.sarray && (xsize(fargs.sarray) > 0)));
 
 	if (data_get_type(src) == DATA_TYPE_LIST) {
 		if (data_list_for_each(src, _foreach_array_entry, &fargs) < 0)
@@ -550,6 +561,8 @@ cleanup:
 		for (int i = 0; fargs.array[i]; i++)
 			free_parser_obj(parser, &fargs.array[i]);
 		xfree(fargs.array);
+	} else if (fargs.sarray) {
+		xfree(fargs.sarray);
 	}
 
 	return rc;
@@ -766,6 +779,15 @@ static int _parser_linked(args_t *args, const parser_t *const array,
 			openapi_append_rel_path(ppath, parser->key);
 	}
 
+	if (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) {
+		log_flag(DATA, "%s: skip parsing removed %s object %s(0x%" PRIxPTR ") via parser %s(0x%" PRIxPTR ")",
+			__func__, set_source_path(&path, args, ppath),
+			parser->obj_type_string, (uintptr_t) dst,
+			parser->type_string, (uintptr_t) src);
+		rc = SLURM_SUCCESS;
+		goto cleanup;
+	}
+
 	if (!src) {
 		if (parser->required) {
 			if ((rc = on_error(PARSING, parser->type, args,
@@ -882,12 +904,13 @@ static void _parse_check_openapi(const parser_t *const parser, data_t *src,
 	 */
 	on_warn(PARSING, parser->type, args,
 		set_source_path(&path, args, parent_path), __func__,
-		"Expected OpenAPI type=%s%s%s (Slurm type=%s) but got OpenAPI type=%s%s%s (Slurm type=%s)",
+		"Expected OpenAPI type=%s%s%s (Slurm type=%s) but got OpenAPI type=%s%s%s (Slurm type=%s): %pd",
 		oas_type, (oas_format ? " format=" : ""),
 		(oas_format ? oas_format : ""),
 		data_type_to_string(oas_data_type), found_type,
 		(found_format ? " format=" : ""),
-		(found_format ? found_format : ""), data_get_type_string(src));
+		(found_format ? found_format : ""), data_get_type_string(src),
+		src);
 
 	xfree(path);
 }
@@ -952,6 +975,12 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 		 parser->type_string, (uintptr_t) parser);
 
 	switch (parser->model) {
+	case PARSER_MODEL_REMOVED:
+		if (data_get_type(src) != DATA_TYPE_NULL)
+			on_warn(PARSING, parser->type, args, path, __func__,
+				"Ignoring value for removed parser");
+		rc = SLURM_SUCCESS;
+		break;
 	case PARSER_MODEL_FLAG_ARRAY:
 		verify_parser_not_sliced(parser);
 		rc = _parse_flag(dst, parser, src, args, parent_path);
@@ -1026,6 +1055,9 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 	case PARSER_MODEL_ARRAY_SKIP_FIELD:
 		fatal_abort("%s: skip model not allowed %u",
 			    __func__, parser->model);
+	case PARSER_MODEL_ARRAY_REMOVED_FIELD:
+		fatal_abort("%s: removed model not allowed %u",
+			    __func__, parser->model);
 	case PARSER_MODEL_INVALID:
 	case PARSER_MODEL_MAX:
 		fatal_abort("%s: invalid model %u", __func__, parser->model);
@@ -1112,6 +1144,9 @@ static void _dump_flag_bit_array_flag(args_t *args, void *src, data_t *dst,
 {
 	bool found;
 
+	if (bit->hidden)
+		return;
+
 	if (bit->type == FLAG_BIT_TYPE_BIT)
 		found = _match_flag_bit(parser, src, bit, *used_equal_bits);
 	else if (bit->type == FLAG_BIT_TYPE_EQUAL)
@@ -1256,7 +1291,12 @@ static int _dump_pointer(const parser_t *const parser, void *src, data_t *dst,
 	const parser_t *pt = find_parser_by_type(parser->pointer_type);
 	void **ptr = src;
 
-	if (!*ptr && !is_complex_mode(args)) {
+	if (!*ptr) {
+		if (is_complex_mode(args)) {
+			xassert(data_get_type(dst) == DATA_TYPE_NULL);
+			return SLURM_SUCCESS;
+		}
+
 		/* Fully resolve pointer on NULL to use correct model */
 		while (pt->pointer_type)
 			pt = find_parser_by_type(pt->pointer_type);
@@ -1339,6 +1379,59 @@ static int _dump_nt_array(const parser_t *const parser, void *src, data_t *dst,
 	return rc;
 }
 
+static void _dump_removed(const parser_t *parser, data_t *dst, args_t *args)
+{
+	if (is_complex_mode(args)) {
+		data_set_null(dst);
+		return;
+	}
+
+	while ((parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) ||
+	       parser->pointer_type) {
+		while (parser->pointer_type)
+			parser = find_parser_by_type(parser->pointer_type);
+
+		while (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD)
+			parser = find_parser_by_type(parser->type);
+	}
+
+	xassert(parser->model != PARSER_MODEL_ARRAY_REMOVED_FIELD);
+	xassert(parser->model !=
+		PARSER_MODEL_ARRAY_LINKED_EXPLODED_FLAG_ARRAY_FIELD);
+	xassert(parser->model > PARSER_MODEL_INVALID);
+	xassert(parser->model < PARSER_MODEL_MAX);
+
+	switch (parser->obj_openapi) {
+	case OPENAPI_FORMAT_INT:
+	case OPENAPI_FORMAT_INT32:
+	case OPENAPI_FORMAT_INT64:
+		data_set_int(dst, 0);
+		break;
+	case OPENAPI_FORMAT_NUMBER:
+	case OPENAPI_FORMAT_FLOAT:
+	case OPENAPI_FORMAT_DOUBLE:
+		data_set_float(dst, 0);
+		break;
+	case OPENAPI_FORMAT_STRING:
+	case OPENAPI_FORMAT_PASSWORD:
+		data_set_string(dst, "");
+		break;
+	case OPENAPI_FORMAT_BOOL:
+		data_set_bool(dst, false);
+	case OPENAPI_FORMAT_OBJECT:
+		data_set_dict(dst);
+		break;
+	case OPENAPI_FORMAT_ARRAY:
+		data_set_list(dst);
+		break;
+	case OPENAPI_FORMAT_MAX:
+	case OPENAPI_FORMAT_INVALID:
+		/* Should never happen but avoid crashing clients */
+		xassert(false);
+		data_set_null(dst);
+	};
+}
+
 static int _dump_linked(args_t *args, const parser_t *const array,
 			const parser_t *const parser, void *src, data_t *dst)
 {
@@ -1372,7 +1465,24 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 			 (uintptr_t) src, array->field_name,
 			 array->ptr_offset, (uintptr_t) dst,
 			 array->key, (uintptr_t) dst);
-		goto cleanup;
+		return SLURM_SUCCESS;
+	}
+
+	if (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) {
+		const parser_t *const rparser =
+			find_parser_by_type(parser->type);
+
+		log_flag(DATA, "removed: %s parser %s->%s(0x%" PRIxPTR ") for %s(0x%" PRIxPTR ") for data(0x%" PRIxPTR ")/%s(0x%" PRIxPTR ")",
+			 parser->obj_type_string,
+			 array->type_string,
+			 parser->type_string, (uintptr_t)
+			 parser, array->obj_type_string,
+			 (uintptr_t) src, (uintptr_t) dst,
+			 array->key, (uintptr_t) dst);
+
+		_dump_removed(rparser, dst, args);
+
+		return SLURM_SUCCESS;
 	}
 
 	if (parser->model ==
@@ -1384,6 +1494,10 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 
 		for (int i = 0; i < parser->flag_bit_array_count; i++) {
 			const flag_bit_t *bit = &parser->flag_bit_array[i];
+
+			if (bit->hidden)
+				continue;
+
 			data_t *bit_dst = data_define_dict_path(dst, bit->name);
 
 			xassert(src);
@@ -1391,7 +1505,7 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 						  bit, true, &used_equal_bits);
 		}
 
-		goto cleanup;
+		return SLURM_SUCCESS;
 	}
 
 	xassert(parser->model == PARSER_MODEL_ARRAY_LINKED_FIELD);
@@ -1412,7 +1526,6 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 		 array->ptr_offset, (uintptr_t) dst, array->key,
 		 (uintptr_t) dst);
 
-cleanup:
 	return rc;
 }
 
@@ -1462,6 +1575,10 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 		goto done;
 
 	switch (parser->model) {
+	case PARSER_MODEL_REMOVED:
+		_dump_removed(parser, dst, args);
+		rc = SLURM_SUCCESS;
+		break;
 	case PARSER_MODEL_FLAG_ARRAY:
 		verify_parser_not_sliced(parser);
 		xassert((data_get_type(dst) == DATA_TYPE_NULL) ||
@@ -1532,6 +1649,9 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 	case PARSER_MODEL_ARRAY_SKIP_FIELD:
 		fatal_abort("%s: skip model not allowed %u",
 			    __func__, parser->model);
+	case PARSER_MODEL_ARRAY_REMOVED_FIELD:
+		fatal_abort("%s: removed model not allowed %u",
+			    __func__, parser->model);
 	case PARSER_MODEL_INVALID:
 	case PARSER_MODEL_MAX:
 		fatal_abort("%s: invalid model %u", __func__, parser->model);
