@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  job.c - Slurm REST API accounting job http operations handlers
  *****************************************************************************
- *  Copyright (C) 2020 SchedMD LLC.
- *  Written by Nathan Rini <nate@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -56,132 +55,28 @@
 #include "src/common/xstring.h"
 
 #include "src/slurmrestd/operations.h"
-#include "src/slurmrestd/plugins/openapi/dbv0.0.38/api.h"
+#include "src/slurmrestd/plugins/openapi/dbv0.0.39/api.h"
+
+#define MAGIC_FOREACH_JOB 0xf8aefef3
+#define MAGIC_FOREACH_SEARCH 0xf9aeaef3
+
+static int _add_list_job_state(List char_list, char *values);
 
 /* typedef for adding a function to add a char* to a List */
 typedef int (*add_list_t) (List char_list, char *values);
 
-#define MAGIC_FOREACH_JOB 0xf8aefef3
 typedef struct {
-	int magic;
+	int magic; /* MAGIC_FOREACH_JOB */
 	data_t *jobs;
-	List tres_list;
-	List qos_list;
-	List assoc_list;
+	ctxt_t *ctxt;
 } foreach_job_t;
 
-static int _foreach_job(void *x, void *arg)
-{
-	slurmdb_job_rec_t *job = x;
-	foreach_job_t *args = arg;
-	parser_env_t penv = {
-		.g_qos_list = args->qos_list,
-		.g_tres_list = args->tres_list,
-		.g_assoc_list = args->assoc_list,
-	};
-
-	xassert(args->magic == MAGIC_FOREACH_JOB);
-
-	if (dump(PARSE_JOB, job, data_set_dict(data_list_append(args->jobs)),
-		 &penv))
-		return -1;
-	else
-		return 1;
-}
-
 typedef struct {
-	data_t *errors;
+	int magic; /* MAGIC_FOREACH_SEARCH */
+	ctxt_t *ctxt;
 	slurmdb_job_cond_t *job_cond;
 } foreach_query_search_t;
 
-data_for_each_cmd_t _foreach_list_entry(data_t *data, void *arg)
-{
-	List list = arg;
-
-	if (data_convert_type(data, DATA_TYPE_STRING) != DATA_TYPE_STRING)
-		return DATA_FOR_EACH_FAIL;
-
-	if (slurm_addto_char_list(list, data_get_string(data)) < 1)
-		return DATA_FOR_EACH_FAIL;
-
-	return DATA_FOR_EACH_CONT;
-}
-
-static int _parse_csv_list(data_t *src, const char *key, List *list,
-			   data_t *errors, add_list_t add_to)
-{
-	if (!*list)
-		*list = list_create(xfree_ptr);
-
-	if (data_get_type(src) == DATA_TYPE_LIST) {
-		if (data_list_for_each(src, _foreach_list_entry, *list) < 0)
-			return resp_error(errors, ESLURM_REST_INVALID_QUERY,
-					  "error parsing CSV in form of list",
-					  key);
-
-		return SLURM_SUCCESS;
-	}
-
-	if (data_convert_type(src, DATA_TYPE_STRING) != DATA_TYPE_STRING)
-		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				  "format must be a string", key);
-
-	if (add_to(*list, data_get_string(src)) < 1)
-		return resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				  "Unable to parse CSV list", key);
-
-	return SLURM_SUCCESS;
-}
-
-/*
- * Convert job state to numeric job state
- * The return value is the same as slurm_addto_char_list():
- *   the number of items added (zero on failure)
- */
-static int _add_list_job_state(List char_list, char *values)
-{
-	int rc = 0;
-	char *last = NULL, *vdup, *value;
-
-	vdup = xstrdup(values);
-	value = strtok_r(vdup, ",", &last);
-	while (value) {
-		char *id_str;
-		uint32_t id = NO_VAL;
-
-		if (isdigit(value[0])) {
-			unsigned long id_ul;
-			errno = 0;
-			id_ul = slurm_atoul(value);
-			/*
-			 * Since zero is a valid value, we have to check if
-			 * errno is also set to know if it was an error.
-			 */
-			if ((!id_ul && errno) || (id_ul == ULONG_MAX))
-				break;
-			id = (uint32_t) id_ul;
-		} else {
-			if ((id = job_state_num(value)) == NO_VAL)
-				break;
-			else
-				id = JOB_STATE_BASE & id;
-		}
-
-		if (id >= JOB_END) {
-			break;
-		}
-
-		id_str = xstrdup_printf("%u", id);
-		rc = slurm_addto_char_list(char_list, id_str);
-		xfree(id_str);
-
-		value = strtok_r(NULL, ",", &last);
-	}
-	xfree(vdup);
-
-	return rc;
-}
-
 typedef struct {
 	char *field;
 	int offset;
@@ -214,6 +109,7 @@ typedef struct {
 	int offset;
 	add_list_t add_to;
 } csv_list_t;
+
 static const csv_list_t csv_lists[] = {
 	{
 		"account",
@@ -275,6 +171,11 @@ static const csv_list_t csv_lists[] = {
 		offsetof(slurmdb_job_cond_t, state_list),
 		_add_list_job_state
 	},
+	{
+		"users",
+		offsetof(slurmdb_job_cond_t, userid_list),
+		slurm_addto_char_list
+	},
 	{
 		"wckey",
 		offsetof(slurmdb_job_cond_t, wckey_list),
@@ -282,6 +183,108 @@ static const csv_list_t csv_lists[] = {
 	},
 };
 
+static int _foreach_job(void *x, void *arg)
+{
+	slurmdb_job_rec_t *job = x;
+	foreach_job_t *args = arg;
+	data_t *jobs = data_list_append(args->jobs);
+
+	xassert(args->magic == MAGIC_FOREACH_JOB);
+	xassert(data_get_type(args->jobs) == DATA_TYPE_LIST);
+
+	if (DATA_DUMP(args->ctxt->parser, JOB, *job, jobs))
+		return -1;
+	else
+		return 1;
+}
+
+static data_for_each_cmd_t _foreach_list_entry(data_t *data, void *arg)
+{
+	List list = arg;
+
+	if (data_convert_type(data, DATA_TYPE_STRING) != DATA_TYPE_STRING)
+		return DATA_FOR_EACH_FAIL;
+
+	if (slurm_addto_char_list(list, data_get_string(data)) < 1)
+		return DATA_FOR_EACH_FAIL;
+
+	return DATA_FOR_EACH_CONT;
+}
+
+static int _parse_csv_list(data_t *src, const char *key, List *list,
+			   ctxt_t *ctxt, add_list_t add_to)
+{
+	if (!*list)
+		*list = list_create(xfree_ptr);
+
+	if (data_get_type(src) == DATA_TYPE_LIST) {
+		if (data_list_for_each(src, _foreach_list_entry, *list) < 0)
+			return resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+					  "error parsing CSV in form of list");
+
+		return SLURM_SUCCESS;
+	}
+
+	if (data_convert_type(src, DATA_TYPE_STRING) != DATA_TYPE_STRING)
+		return resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				  "format must be a string");
+
+	if (add_to(*list, data_get_string(src)) < 1)
+		return resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				  "Unable to parse CSV list");
+
+	return SLURM_SUCCESS;
+}
+
+/*
+ * Convert job state to numeric job state
+ * The return value is the same as slurm_addto_char_list():
+ *   the number of items added (zero on failure)
+ */
+static int _add_list_job_state(List char_list, char *values)
+{
+	int rc = 0;
+	char *last = NULL, *vdup, *value;
+
+	vdup = xstrdup(values);
+	value = strtok_r(vdup, ",", &last);
+	while (value) {
+		char *id_str;
+		uint32_t id = NO_VAL;
+
+		if (isdigit(value[0])) {
+			unsigned long id_ul;
+			errno = 0;
+			id_ul = slurm_atoul(value);
+			/*
+			 * Since zero is a valid value, we have to check if
+			 * errno is also set to know if it was an error.
+			 */
+			if ((!id_ul && errno) || (id_ul == ULONG_MAX))
+				break;
+			id = (uint32_t) id_ul;
+		} else {
+			if ((id = job_state_num(value)) == NO_VAL)
+				break;
+			else
+				id = JOB_STATE_BASE & id;
+		}
+
+		if (id >= JOB_END) {
+			break;
+		}
+
+		id_str = xstrdup_printf("%u", id);
+		rc = slurm_addto_char_list(char_list, id_str);
+		xfree(id_str);
+
+		value = strtok_r(NULL, ",", &last);
+	}
+	xfree(vdup);
+
+	return rc;
+}
+
 static data_for_each_cmd_t _foreach_step(data_t *data, void *arg)
 {
 	List list = arg;
@@ -300,21 +303,21 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 						 void *arg)
 {
 	foreach_query_search_t *args = arg;
-	data_t *errors = args->errors;
+	ctxt_t *ctxt = args->ctxt;
+
+	xassert(args->magic == MAGIC_FOREACH_SEARCH);
 
 	if (!xstrcasecmp("start_time", key)) {
 		if (args->job_cond->flags & JOBCOND_FLAG_NO_DEFAULT_USAGE) {
-			resp_error(
-				errors, ESLURM_REST_INVALID_QUERY,
-				"start_time and submit_time are mutually exclusive",
-				key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "start_time and submit_time are mutually exclusive");
 			return DATA_FOR_EACH_FAIL;
 		}
 
 		if (data_convert_type(data, DATA_TYPE_STRING) !=
 		    DATA_TYPE_STRING) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "Time format must be a string", key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "Time format must be a string");
 			return DATA_FOR_EACH_FAIL;
 		}
 
@@ -322,8 +325,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 							 1);
 
 		if (!args->job_cond->usage_start) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "Unable to parse time format", key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "Unable to parse time format");
 			return DATA_FOR_EACH_FAIL;
 		}
 
@@ -333,8 +336,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 	if (!xstrcasecmp("end_time", key)) {
 		if (data_convert_type(data, DATA_TYPE_STRING) !=
 		    DATA_TYPE_STRING) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "Time format must be a string", key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "Time format must be a string");
 			return DATA_FOR_EACH_FAIL;
 		}
 
@@ -342,8 +345,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 						       1);
 
 		if (!args->job_cond->usage_end) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "Unable to parse time format", key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "Unable to parse time format");
 			return DATA_FOR_EACH_FAIL;
 		}
 
@@ -352,17 +355,15 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 
 	if (!xstrcasecmp("submit_time", key)) {
 		if (args->job_cond->usage_start) {
-			resp_error(
-				errors, ESLURM_REST_INVALID_QUERY,
-				"start_time and submit_time are mutually exclusive",
-				key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				"start_time and submit_time are mutually exclusive");
 			return DATA_FOR_EACH_FAIL;
 		}
 
 		if (data_convert_type(data, DATA_TYPE_STRING) !=
 		    DATA_TYPE_STRING) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "Time format must be a string", key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "Time format must be a string");
 			return DATA_FOR_EACH_FAIL;
 		}
 
@@ -370,8 +371,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 							 1);
 
 		if (!args->job_cond->usage_start) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "Unable to parse time format", key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "Unable to parse time format");
 			return DATA_FOR_EACH_FAIL;
 		}
 
@@ -383,8 +384,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 	if (!xstrcasecmp("node", key)) {
 		if (data_convert_type(data, DATA_TYPE_STRING) !=
 		    DATA_TYPE_STRING) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "format must be a string", key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "format must be a string");
 			return DATA_FOR_EACH_FAIL;
 		}
 
@@ -402,10 +403,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 		if (data_get_type(data) == DATA_TYPE_LIST) {
 			if (data_list_for_each(data, _foreach_step,
 					       args->job_cond->step_list) < 0) {
-				(void) resp_error(
-					errors, ESLURM_REST_INVALID_QUERY,
-					"error parsing steps in form of list",
-					key);
+				resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+					"error parsing steps in form of list");
 				return DATA_FOR_EACH_FAIL;
 			}
 
@@ -414,8 +413,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 
 		if (data_convert_type(data, DATA_TYPE_STRING) !=
 		    DATA_TYPE_STRING) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "format must be a string", key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "format must be a string");
 			return DATA_FOR_EACH_FAIL;
 		}
 
@@ -423,8 +422,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 				      data_get_string(data));
 
 		if (!list_count(args->job_cond->step_list)) {
-			resp_error(errors, ESLURM_REST_INVALID_QUERY,
-				   "Unable to parse job/step format", key);
+			resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+				   "Unable to parse job/step format");
 			return DATA_FOR_EACH_FAIL;
 		}
 
@@ -436,10 +435,36 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 			List *list = (((void *) args->job_cond) +
 				      csv_lists[i].offset);
 
-			if (_parse_csv_list(data, key, list, errors,
+			if (_parse_csv_list(data, key, list, ctxt,
 					    csv_lists[i].add_to))
 				return DATA_FOR_EACH_FAIL;
 
+			if (!xstrcasecmp("groups", key)) {
+				List list2 = list_create(xfree_ptr);
+				if (list_for_each_ro(*list, groupname_to_gid,
+						     list2) < 0) {
+					list_destroy(list2);
+					resp_error(ctxt,
+						   ESLURM_REST_MISSING_GID, key,
+						   "error resolving GID from group name");
+					return DATA_FOR_EACH_FAIL;
+				}
+				list_destroy(*list);
+				*list = list2;
+			} else if (!xstrcasecmp("users", key)) {
+				List list2 = list_create(xfree_ptr);
+				if (list_for_each_ro(*list, username_to_uid,
+						     list2) < 0) {
+					list_destroy(list2);
+					resp_error(ctxt,
+						   ESLURM_REST_MISSING_UID, key,
+						   "error resolving UID from user name");
+					return DATA_FOR_EACH_FAIL;
+				}
+				list_destroy(*list);
+				*list = list2;
+			}
+
 			return DATA_FOR_EACH_CONT;
 		}
 	}
@@ -448,8 +473,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 		if (!xstrcasecmp(flags[i].field, key)) {
 			if (data_convert_type(data, DATA_TYPE_BOOL) !=
 			    DATA_TYPE_BOOL) {
-				resp_error(errors, ESLURM_REST_INVALID_QUERY,
-					   "must be an Boolean", key);
+				resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+					   "must be an Boolean");
 				return DATA_FOR_EACH_FAIL;
 			}
 
@@ -469,8 +494,8 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 
 			if (data_convert_type(data, DATA_TYPE_INT_64) !=
 			    DATA_TYPE_INT_64) {
-				resp_error(errors, ESLURM_REST_INVALID_QUERY,
-					   "must be an integer", key);
+				resp_error(ctxt, ESLURM_REST_INVALID_QUERY, key,
+					   "must be an integer");
 				return DATA_FOR_EACH_FAIL;
 			}
 
@@ -480,33 +505,21 @@ static data_for_each_cmd_t _foreach_query_search(const char *key,
 		}
 	}
 
-	resp_error(errors, ESLURM_REST_INVALID_QUERY, "Unknown Query field",
-		   NULL);
+	resp_error(ctxt, ESLURM_REST_INVALID_QUERY, __func__,
+		   "unexpected Query field: %s", key);
 	return DATA_FOR_EACH_FAIL;
 }
 
-static int _dump_jobs(const char *context_id, http_request_method_t method,
-		      data_t *parameters, data_t *query, int tag, data_t *resp,
-		      void *auth, data_t *errors, slurmdb_job_cond_t *job_cond)
+static void _dump_jobs(ctxt_t *ctxt, slurmdb_job_cond_t *job_cond)
 {
-	int rc = SLURM_SUCCESS;
-	slurmdb_assoc_cond_t assoc_cond = {
-		.with_deleted = 1,
-		.without_parent_info = 1,
-		.without_parent_limits = 1,
-	};
-	slurmdb_qos_cond_t qos_cond = {
-		.with_deleted = 1,
-	};
-	slurmdb_tres_cond_t tres_cond = {
-		.with_deleted = 1,
-	};
 	foreach_job_t args = {
 		.magic = MAGIC_FOREACH_JOB,
-		.jobs = data_set_list(data_key_set(resp, "jobs")),
+		.ctxt = ctxt,
 	};
 	List jobs = NULL;
 
+	args.jobs = data_set_list(data_key_set(ctxt->resp, "jobs"));
+
 	/* set cluster by default if not specified */
 	if (job_cond &&
 	    (!job_cond->cluster_list ||
@@ -517,22 +530,13 @@ static int _dump_jobs(const char *context_id, http_request_method_t method,
 			    xstrdup(slurm_conf.cluster_name));
 	}
 
-	if (!db_query_list(errors, auth, &jobs, slurmdb_jobs_get, job_cond) &&
-	    !db_query_list(errors, auth, &args.assoc_list,
-			   slurmdb_associations_get, &assoc_cond) &&
-	    !db_query_list(errors, auth, &args.qos_list, slurmdb_qos_get,
-			   &qos_cond) &&
-	    !db_query_list(errors, auth, &args.tres_list, slurmdb_tres_get,
-			   &tres_cond) &&
-	    (list_for_each(jobs, _foreach_job, &args) < 0))
-		rc = ESLURM_DATA_CONV_FAILED;
+	if (!db_query_list(ctxt, &jobs, slurmdb_jobs_get, job_cond) && jobs)
+		list_for_each(jobs, _foreach_job, &args);
 
-	FREE_NULL_LIST(args.tres_list);
-	FREE_NULL_LIST(args.qos_list);
-	FREE_NULL_LIST(args.assoc_list);
 	FREE_NULL_LIST(jobs);
 
-	return rc;
+	if (job_cond)
+		FREE_NULL_LIST(job_cond->cluster_list);
 }
 
 /* based on get_data() in sacct/options.c */
@@ -540,9 +544,16 @@ extern int op_handler_jobs(const char *context_id, http_request_method_t method,
 			   data_t *parameters, data_t *query, int tag,
 			   data_t *resp, void *auth, data_parser_t *parser)
 {
-	data_t *errors = populate_response_format(resp);
+	ctxt_t *ctxt = init_connection(context_id, method, parameters, query,
+				       tag, resp, auth);
 
-	if (query && data_get_dict_length(query)) {
+	if (ctxt->rc) {
+		/* no-op - already logged */
+	} else if (method != HTTP_REQUEST_GET) {
+		resp_error(ctxt, ESLURM_REST_INVALID_QUERY, __func__,
+			   "Unsupported HTTP method requested: %s",
+			   get_http_method_string(method));
+	} else if (query && data_get_dict_length(query)) {
 		slurmdb_job_cond_t job_cond = {
 			/*
 			 * default to grabbing all information
@@ -552,18 +563,19 @@ extern int op_handler_jobs(const char *context_id, http_request_method_t method,
 			.db_flags = SLURMDB_JOB_FLAG_NOTSET,
 		};
 		foreach_query_search_t args = {
-			.errors = errors,
+			.magic = MAGIC_FOREACH_SEARCH,
+			.ctxt = ctxt,
 			.job_cond = &job_cond,
 		};
 
-		if (data_dict_for_each(query, _foreach_query_search, &args) < 0)
-			return SLURM_ERROR;
+		if (data_dict_for_each(query, _foreach_query_search, &args) >=
+		    0)
+			_dump_jobs(ctxt, &job_cond);
+	} else {
+		_dump_jobs(ctxt, NULL);
+	}
 
-		return _dump_jobs(context_id, method, parameters, query, tag,
-				  resp, auth, errors, &job_cond);
-	} else
-		return _dump_jobs(context_id, method, parameters, query, tag,
-				  resp, auth, errors, NULL);
+	return fini_connection(ctxt);
 }
 
 /* based on get_data() in sacct/options.c */
@@ -571,32 +583,35 @@ static int _op_handler_job(const char *context_id, http_request_method_t method,
 			   data_t *parameters, data_t *query, int tag,
 			   data_t *resp, void *auth, data_parser_t *parser)
 {
-	int rc = SLURM_SUCCESS;
-	data_t *errors = populate_response_format(resp);
 	char *jobid;
 	slurmdb_job_cond_t job_cond = {
 		.flags = (JOBCOND_FLAG_DUP | JOBCOND_FLAG_NO_TRUNC),
 		.db_flags = SLURMDB_JOB_FLAG_NOTSET,
 	};
+	ctxt_t *ctxt = init_connection(context_id, method, parameters, query,
+				       tag, resp, auth);
 
-	if ((jobid = get_str_param("job_id", errors, parameters))) {
+	if (ctxt->rc) {
+		/* no-op - already logged */
+	} else if (method != HTTP_REQUEST_GET) {
+		resp_error(ctxt, ESLURM_REST_INVALID_QUERY, __func__,
+			   "Unsupported HTTP method requested: %s",
+			   get_http_method_string(method));
+	} else if ((jobid = get_str_param("job_id", ctxt))) {
 		job_cond.step_list = list_create(slurm_destroy_selected_step);
 		slurm_addto_step_list(job_cond.step_list, jobid);
-	} else
-		rc = ESLURM_REST_INVALID_QUERY;
 
-	if (!rc)
-		rc = _dump_jobs(context_id, method, parameters, query, tag,
-				resp, auth, errors, &job_cond);
+		_dump_jobs(ctxt, &job_cond);
+	}
 
 	FREE_NULL_LIST(job_cond.step_list);
-	return rc;
+	return fini_connection(ctxt);
 }
 
 extern void init_op_job(void)
 {
-	bind_operation_handler("/slurmdb/v0.0.38/jobs/", op_handler_jobs, 0);
-	bind_operation_handler("/slurmdb/v0.0.38/job/{job_id}", _op_handler_job,
+	bind_operation_handler("/slurmdb/v0.0.39/jobs/", op_handler_jobs, 0);
+	bind_operation_handler("/slurmdb/v0.0.39/job/{job_id}", _op_handler_job,
 			       0);
 }
 
